/**
 * mb-js-bridge-umd
 *
 */

import 'url-search-params-polyfill'
import './utils/objectAssignPolyfill'

import openPage from './openPage'
import callBridge from './utils/callBridge'
import ajax from './utils/ajax'
import webCommon from './utils/web-common'
import forceResolveHTTPS from './utils/forceResolveHTTPS'

callBridge('INIT', null, false);

var Bridge = {

    /**
     * 当前 js bridge 版本号
     */
    version: process.env.VERSION,

    /**
     * 打开页面
     */
    openPage: openPage,

    /**
     * Ajax 请求方法
     */
    ajax: ajax,

    /**
     * 获取 API 域名，prefix 为空时返回完整域名，否则将用 prefix 替换 ciqapi 的部分
     * @param {string} prefix 前缀
     * @returns {string} 返回重新拼接后的域名
     */
    getApiDomain: function (prefix) {
        var session = this.getSession();
        if (!session) {
            return '';
        }
        return webCommon.getApiDomainByPrefix(prefix, session.domain);
    },

    /**
     * 获取用户会话信息，成功返回会话信息对象，失败返回 false
     *
     * @since v4.0
     * @returns {{appName,sysType,networkType,SSID,MAC,appVersion,sysVersion,sessionId,lang,deviceId,domain}|boolean}
     */
    getSession: function () {
        var s = callBridge('GET_USER');
        if (s.domain) s.domain = forceResolveHTTPS(s.domain);
        return s;
    },

    /**
     * 获取用户个人信息
     *
     * @since v4.0
     * @return {{userId,companyId,sessionId,name,avatar,level,golds,stars,diamonds,lifePoint,phone}|boolean}
     */
    getUserInfo: function () {
        return callBridge('GET_USER_INFO')
    },

    /**
     * 重登录，当用户信息过期时使用
     */
    sessionInvalid: function () {
        callBridge('SESSION_INVALID')
    },

    /**
     *
     */
    setCallbackArgs: function (data) {
        callBridge('CALLBACK_ARGUMENTS', {
            arguments: data
        })
    },

    /**
     * 关闭 web 页面
     */
    closePage: function () {
        callBridge('CLOSE_WINDOW');
    },

    /**
     * 设置右上角操作按钮
     *
     * @since v5.0.6
     * @param buttonName
     * @param callback
     */
    setOperationButton: function (buttonName, callback) {
        callBridge('SET_RIGHT_BUTTON', {
            buttonName: buttonName,
            callback: callback
        });
    },

    /**
     * 输出日志到 APP 控制台
     *
     * @deprecated 不再推荐使用
     */
    log: function (obj) {
        callBridge('LOG', obj);
    },

    /**
     * 上传图片
     *
     * @since v4.0.0
     * @param {function(string)} callback
     * @param {string} type 上传类型，默认为 common。支持的值有 avatar / common / album
     */
    uploadImage: function (callback, type) {
        if (typeof callback !== 'function')
            throw new Error('UploadImage callback must be a function');

        callBridge('UPLOAD_PIC', {
            callback: callback,
            type: type || 'common'
        })
    },

    /**
     * 预览图片
     *
     * @since v4.0
     * @param {string|Array} imageUrl
     * @param {number|string} currentPos
     * @returns {*}
     */
    showImage: function (imageUrl, currentPos) {
        // 把 imageUrl 统一处理成数组格式，currentPos 为数组索引
        if (!currentPos) currentPos = 0;

        if (imageUrl instanceof Array) {
            currentPos = typeof currentPos === 'number' ? currentPos : imageUrl.indexOf(currentPos)
        } else {
            imageUrl = [imageUrl]
        }

        callBridge('SHOW_IMAGE', {
            imageUrl: imageUrl,
            currentPos: currentPos
        });
    },

    /**
     * 拍摄短视频并上传
     *
     * @returns {{videoUrl, posterUrl}}
     */
    takeShortVideo: function () {
        return callBridge('TAKE_SHORT_VIDEO');
    },

    /**
     * 获取网络状态
     */
    getNetState: function () {
        return callBridge('NET_STATE');
    },

    /**
     * 获取位置信息
     *
     * @return {object}
     */
    getLocation: function () {
        return callBridge('LOCATION');
    },

    /**
     * 显示加载动画
     */
    showLoading: function() {
        callBridge('OPEN_LOADING')
    },

    /**
     * 隐藏加载动画
     */
    hideLoading: function() {
        callBridge('CLOSE_LOADING');
    },

    /**
     * 隐藏顶部标题栏
     *
     * @since v3.4.0
     */
    hideTitleBar: function () {
        callBridge('HIDE_TITLE_BAR')
    },

    /**
     * 分享
     *
     * @since v3.6.0
     * @param url
     * @param title
     * @param desc
     * @param image
     */
    share: function (url, title, desc, image) {
        return callBridge('SHARE', {
            url: url,
            title: title,
            desc: desc || '',
            image: image || ''
        });
    },

    /**
     * 高级分享，目前仅支持 CIQ
     *
     * @since v5.4.0
     * @param url
     * @param title
     * @param desc
     * @param image
     */
    advancedShare: function(url, title, desc, image) {
        callBridge('PLATFORM_SHARE', {
            url: url,
            title: title,
            desc: desc || '',
            image: image || ''
        });
    },

    /**
     * 事件记录（埋点）
     *
     * @since v4.1.1
     * @param eventName
     * @param props
     * @returns {*}
     */
    eventsStatistic: function (eventName, props) {
        return callBridge('EVENTS_STATISTIC', {
            eventName: eventName,
            properties: props || {}
        })
    },

    /**
     * 事件记录（埋点）
     * @deprecated 此方法用于兼容旧版本 JS Bridge，请使用 eventsStatistic 替换
     * @param event
     * @returns {void}
     */
    track: function (event) {
        callBridge('EVENTS_STATISTIC', event)
    },

    /**
     * 获取解密数据值
     */
    getDecryptValue: function (key, data) {
         return callBridge('DECRYPT_DATA', {
            key: key,
            data: data
        });
    },

    /**
     * 注册在关闭页面时执行的回调（目前仅用于安卓）
     *
     * @param callback
     */
    onClosePage: function (callback) {
        if (typeof callback !== 'function') {
            console.error('onCloseWindow callback must be a function');
            return;
        }

        window._$onCloseWindowEvent = function () {
            callback();
        };

        prompt('JS_BRIDGE_INTERCEPT_CLOSE_WINDOW_EVENT', '{"func":"_$onCloseWindowEvent"}');
    },

    /**
     * 打开选择联系人界面并选取一个联系人
     *
     * @return {boolean|{id, name, avatar}} 选择后选择联系人的基本信息，未选择时返回 false
     */
    getContact: function () {
        return callBridge('SELECT_CONTACT');
    }
};

export default Bridge
