/**
 *
 * @author    Jerry Bendy
 * @since     7/20/2018
 */
import checkAccess from '../utils/checkAccess'
import webCommon from '../utils/web-common';
import { bridgeMethods as webBridgeMethods } from './web'
import getUploadToken from '../utils/getUploadToken'
import ajax from '../utils/ajax'
import dom from '../utils/dom'
import * as mixPanel from '../utils/mixpanel'

var isAccessible = true     // 是否可访问 JS Bridge 资源

var bridgeMethods = {
    INIT: function() {
        if (!checkAccess()) {
            isAccessible = false
            return;
        }

        // 重写 prompt 方法
        window.prompt = function(msg, _default) {
            try {
                if (_default) {
                    _default = JSON.parse(_default)
                }
            } catch (e) {}

            invoke(msg, _default, false)
        }

        // 初始化 MixPanel
        mixPanel.initMixPanel()

        // 在窗体的 load 事件中加载对 DOM 的修改，避免把 JS 脚本放在 head 标签中时 body 元素未加载的问题
        window.addEventListener('load', function() {
            // 加载额外的 CSS 样式
            webCommon.setExtraCSS('mobile')

            // 当前页面未设置字体时，修改默认字体为无衬线字体
            if (!document.body.style.fontFamily) {
                document.body.style.fontFamily = '"Helvetica Neue", Helvetica, "PingFang SC", "Hiragino Sans GB", "Microsoft YaHei", 微软雅黑, Arial, sans-serif'
            }
        })
    },

    JS_BRIDGE_UPLOAD_PIC: function(data) {
        var callback = data.callback,
            type = data.type

        var $file = webCommon.getUploadInput(function(e) {
            var file = e.target.files[0]
            if (!file) {
                return
            }

            webCommon.showLoading()

            // 获取上传 token
            getUploadToken({
                type: type,
                success: function(data) {
                    var upHost = data.action,
                        token = data.qiniuToken,
                        frm = new FormData()

                    // 压缩图片
                    webCommon.imageCompress(file, function(result) {
                        frm.append('token', token);
                        frm.append('file', result, result.name);

                        ajax.post({
                            url: upHost,
                            data: frm,
                            withoutSession: true,  // 不传 session 信息，默认关闭
                            success: function(data) {
                                if (data.status === 200) {
                                    callback(data.data.url)
                                } else {
                                    throw new Error(data.message)
                                }
                            },
                            error: function(err) {
                                console.error(err)
                            },
                            complete: function() {
                                webCommon.hideLoading()
                                $file = null
                            }
                        })
                    })
                },
                error: function(err) {
                    throw new Error(err)
                }
            })
        })

        $file.click()
    },

    JS_BRIDGE_SHOW_IMAGE: function(data) {
        getImagePreviewer(data.imageUrl[data.currentPos])
    },

    /**
     * 重登录，当用户信息过期时使用
     */
    JS_BRIDGE_SESSION_INVALID: function() {
        webCommon.logout()
    },

    JS_BRIDGE_OPEN_PAGE: function(params) {
        var pageName = params.pageName,
            h5Domain = webCommon.getH5Domain()

        switch (pageName) {
            // case 'COURSE':
            //     // 知识库
            // case 'COURSE_SET_DETAIL':
            //     // 课程集
            //     location.href = h5Domain + 'api/functions/ciq-only'
            //     break
            case 'TASK':
                location.href = h5Domain + 'api/task/list'
                break

            case 'TASK_DETAIL':
                location.href = h5Domain + 'api/task/detail?' + new URLSearchParams(params.data)
                break

            case 'COURSE_QUIZ':
                location.href = h5Domain + 'api/course/quiz-gate?courseId=' + params.data.courseId
                break

            case 'COURSE_PAGE':
                location.href = h5Domain + 'api/course/course-page-container?courseSetId=' + params.data.courseSetId + '&courseId=' + params.data.courseId
                break

            case 'ACHIEVEMENT':
                location.href = h5Domain + 'api/achievement'
                break

            default:
                webBridgeMethods.JS_BRIDGE_OPEN_PAGE(params)
        }
    },

    /**
     * 事件记录（埋点）
     */
    JS_BRIDGE_EVENTS_STATISTIC: function(params) {
        mixPanel.trackEvent(params.eventName, params.properties)
    }
}

function getImagePreviewer(imageUrl) {
    var id = '__BRIDGE_PREVIEW',
        $previewer = document.getElementById(id)

    if ($previewer) {
        return $previewer
    }

    $previewer = document.createElement('div');
    $previewer.id = id
    $previewer.setAttribute('class', '__bridge-overlay');
    // 缓存 body 的 overflow 的值，用于隐藏后还原
    $previewer.dataset.bodyOverflow = document.body.style.overflow;

    var $image = document.createElement('img');
    $image.setAttribute('class', '__bridge-image-preview');
    $image.src = imageUrl;

    // 标识是否需要显示加载动画。当图片在 0.3 秒内未能显示时，将会显示加载动画
    var needShowLoading = true;

    $image.addEventListener('load', function() {
        needShowLoading = false;
        webCommon.hideLoading();
    })

    // 等待 0.3 秒，如果未执行 load 事件，将会显示加载动画
    setTimeout(function() {
        if (needShowLoading) {
            webCommon.showLoading()
        }
    }, 300)

    $previewer.appendChild($image);

    $previewer.addEventListener('click', function() {
        dom.fade($previewer, 'out', 100, function() {
            document.body.style.overflow = $previewer.dataset.bodyOverflow;
            $previewer.remove();
        })
    })

    document.body.style.overflow = 'hidden';
    document.body.appendChild($previewer);

    dom.fade($previewer, 'in', 100)

    return $previewer
}

function invoke(key, data) {
    if (!isAccessible) {
        return null;
    }

    if (typeof bridgeMethods[key] === 'function') {
        return bridgeMethods[key](data)
    }

    // 尝试从 web 的方法中寻找对应的处理方式
    if (typeof webBridgeMethods[key] === 'function') {
        return webBridgeMethods[key](data)
    }

    return null;
}

export default invoke
