/**
 *
 * @author    Jerry Bendy
 * @since     4/23/2018
 */
import checkAccess from '../utils/checkAccess'
import webCommon from '../utils/web-common'
import * as mixPanel from '../utils/mixpanel'

var isAccessible = true     // 是否可访问 JS Bridge 资源

var isInFrame = window.parent !== window  // 标识当前是否处于 iframe 中

var isInWebHost = false  // 标识是否位于 web 宿主中


var bridgeMethods = {
    /**
     * 初始化
     */
    INIT: function() {
        if (!checkAccess()) {
            isAccessible = false
            return
        }

        // ping 消息到父页面并接收 pong，以确认是否在 web 端宿主中
        // 生成一个随机数，并将该数字 ping 到宿主中，如果能完能收到 pong 的消自并
        // 且值相同的话， 即可确认是在宿主环境中
        if (isInFrame) {
            var random = Math.floor(Math.random() * 100000000)
            var __receivePong = function(event) {
                if (!event.data || !event.data.type) return

                if (event.data.type === '__pong' && event.data.payload === random) {
                    isInWebHost = true
                }

                window.removeEventListener('message', __receivePong, false)
            }

            window.addEventListener('message', __receivePong, false)

            sendMessage('__ping', random)
        }

        // 重写 prompt 方法
        window.prompt = function(msg, _default) {
            try {
                if (_default) {
                    _default = JSON.parse(_default)
                }
            } catch (e) {}

            invoke(msg, _default, false)
        }

        // 重写 alert 方法
        if (isInFrame) {
            var windowAlert = window.alert

            window.alert = function(message, params) {
                if (isInWebHost) {
                    sendMessage('__alert', {
                        message: message,
                        params: params || {}
                    })
                    console.debug(message)
                } else {
                    windowAlert(message)
                }
            }
        }

        // 初始化 MixPanel
        mixPanel.initMixPanel()

        // 在窗体的 load 事件中加载对 DOM 的修改，避免把 JS 脚本放在 head 标签中时 body 元素未加载的问题
        window.addEventListener('load', function() {
            // 加载额外的 CSS 样式
            webCommon.setExtraCSS('web')

            // 当前页面未设置字体时，修改默认字体为无衬线字体
            if (!document.body.style.fontFamily) {
                document.body.style.fontFamily = '"Helvetica Neue", Helvetica, "PingFang SC", "Hiragino Sans GB", "Microsoft YaHei", 微软雅黑, Arial, sans-serif'
            }

            // 当位于 frame 中时，加载完成后发送页面标题给外部框架
            if (isInFrame) {
                sendMessage('__js-bridge-set-page-title', {
                    title: document.title
                })

                try {
                    // 为 document.title 添加 getter 和 setter，以在页面标题变化时可以自动更新界面上的标题
                    // 加上 try catch 是为避免重复定义 Object.defineProperty 抛出的异常
                    var __title = document.title
                    Object.defineProperty(document, 'title', {
                        set(value) {
                            sendMessage('__js-bridge-set-page-title', {
                                title: value
                            })
                            __title = value
                        },
                        get() {
                            return __title
                        }
                    })
                } catch (e) {}

            }
        })
    },

    /**
     * 获取用户信息
     */
    JS_BRIDGE_GET_USER: function() {
        var cookieData = webCommon.getCookieFields()

        return Object.assign({}, {
            appName: 'ciq3',
            appVersion: 'web',
            sysType: 'web',
            sysVersion: 'unknown',
            deviceId: 'WECHAT',
            sessionId: '',
            lang: 'zh',
            domain: webCommon.getApiDomain(),
            uid: cookieData.userId || ''
        }, cookieData)
    },

    /**
     * 获取用户资料
     */
    JS_BRIDGE_GET_USER_INFO: function() {
        var cookieData = webCommon.getCookieFields()

        return {
            userId: cookieData.userId || '',
            companyId: cookieData.companyId || ''
        }
    },

    /**
     * 打开指定页面
     *
     * @param params
     */
    JS_BRIDGE_OPEN_PAGE: function(params) {
        var pageName = params.pageName,
            data = params.data || {},
            h5Domain = webCommon.getH5Domain()

        switch (pageName) {
            case 'WEB_PAGE':
                location.href = data.url
                break

            case 'COURSE':
                // 知识库
                location.href = h5Domain + 'api/course/courses'
                break

            case 'COURSE_SET_DETAIL':
                // 课程集
                location.href = h5Domain + 'api/course/course-set?courseId=' + data.id
                break

            case 'COURSE_PAGE':
                // 课程学习页面
                sendMessage('__js-bridge-open-course-page', data)
                break

            case 'COURSE_QUIZ':
                // 课程挑战
                sendMessage('__js-bridge-open-course-quiz', data)
                break

            case 'TASK':
                // 任务列表
                location.href = h5Domain + 'api/task/list'
                break

            case 'TASK_DETAIL':
                // 任务详情
                location.href = h5Domain + 'api/task/detail?' + new URLSearchParams(data)
                break
            
            case 'ACHIEVEMENT':
                location.href = h5Domain + 'api/achievement'
                break

            case 'USERCENTER':
                // 用户中心
                location.href = h5Domain + 'api/user/profile?userId=' + data.userId
                break

            case 'PERSON_CHAT':
                // 单聊
                sendMessage('__js-bridge-open-person-chat', {
                    userId: data.fId,
                    realName: data.fName
                })
                break
        }
    },

    /**
     * 显示加载动画（不包含文本）
     */
    JS_BRIDGE_OPEN_LOADING: function() {
        webCommon.showLoading()
    },

    /**
     * 隐藏加载动画
     */
    JS_BRIDGE_CLOSE_LOADING: function() {
        webCommon.hideLoading()
    },

    /**
     * 上传图片
     * @param data
     */
    JS_BRIDGE_UPLOAD_PIC: function(data) {
        var callback = data.callback,
            type = data.type

        function _postMessageCallback(event) {
            if (!event.data || !event.data.type || event.data.type !== '__js-bridge-upload-image-callback') {
                return
            }

            window.removeEventListener('message', _postMessageCallback, false)
            callback(event.data.payload.imgUrl)
        }

        window.addEventListener('message', _postMessageCallback, false)

        sendMessage('__js-bridge-upload-image', {
            type: type
        })
    },

    /**
     * 图片预览
     * @param data
     */
    JS_BRIDGE_SHOW_IMAGE: function(data) {
        sendMessage('__js-bridge-image-preview', {
            images: data.imageUrl,
            currentPos: data.currentPos
        })
    },

    /**
     * 分享
     * @param {{url, title, desc, image}} params
     */
    JS_BRIDGE_SHARE: function(params) {
        sendMessage('__js-bridge-share', params)
    },

    /**
     * 高级分享（处理方式同分享接口）
     * @param params
     */
    JS_BRIDGE_PLATFORM_SHARE: function(params) {
        sendMessage('__js-bridge-share', params)
    },

    /**
     * 关闭 web 页面
     */
    JS_BRIDGE_CLOSE_WINDOW: function() {
        sendMessage('__js-bridge-close-page')
    },

    /**
     * 重登录，当用户信息过期时使用
     */
    JS_BRIDGE_SESSION_INVALID: function() {
        sendMessage('__js-bridge-logout')
    },

    /**
     * 事件记录（埋点）
     */
    JS_BRIDGE_EVENTS_STATISTIC: function(params) {
        mixPanel.trackEvent(params.eventName, params.properties)
    }
}

function sendMessage(type, payload) {
    window.parent.postMessage({
        type: type,
        payload: payload || {}
    }, '*')
}

function invoke(key, data) {
    if (typeof bridgeMethods[key] === 'function') {

        if (!isAccessible) {
            return null;
        }

        return bridgeMethods[key](data)
    }

    return null;
}

export default invoke

export { bridgeMethods }
