/**
 *
 * @author    Jerry Bendy
 * @since     2/26/2018
 */
/* global wx */

import loadScript from 'scriptjs'
import ajax from '../utils/ajax'
import checkAccess from '../utils/checkAccess'
import webCommon from '../utils/web-common'
import callBridge from '../utils/callBridge'
import * as mixPanel from '../utils/mixpanel'
import { bridgeMethods as webBridgeMethods } from './web'

var isWechatReady = false,  // 微信环境是否初始化完成
    isAccessible = true,   // 是否可访问 JS Bridge 资源
    callStack = [],   // 初始化完成前的调用均缓存至此
    wechatVersion = '-'   // 微信版本号

var bridgeMethods = {
    /**
     * 初始化
     */
    INIT: function() {
        if (!checkAccess()) {
            isAccessible = false;
            return;
        }

        // 从 UserAgent 中获取当前微信版本号
        var v = navigator.userAgent.match(/MicroMessenger\/([\d.]+)\s?/)
        if (v && v[1]) {
            wechatVersion = v[1]
        }

        // 加载微信 JS-sdk 文件
        loadScript('//res.wx.qq.com/open/js/jweixin-1.2.0.js', function() {
            // 微信配置
            ajax.get({
                url: webCommon.getH5Domain() + 'api/wechat/get-js-signature?url=' + encodeURIComponent(location.href),
                success: function(data) {
                    wx.config(data)

                    wx.ready(function() {
                        isWechatReady = true

                        // 隐藏所有非基础按钮
                        wx.hideAllNonBaseMenuItem();

                        wx.hideMenuItems({
                            menuList: [
                                "menuItem:share:appMessage",
                                "menuItem:share:timeline"
                            ]
                        })

                        // 执行保存在 stack 中的调用
                        while (callStack.length) {
                            var c = callStack.shift()
                            invoke.apply(null, c);
                        }
                    })

                    wx.error(function(res) {
                        console.error(res)
                    })
                }
            })
        })

        // 重写 prompt 方法
        window.prompt = function(msg, _default) {
            try {
                if (_default) {
                    _default = JSON.parse(_default)
                }
            } catch (e) {}

            invoke(msg, _default, false)
        }

        // 初始化 MixPanel
        mixPanel.initMixPanel()

        // 在窗体的 load 事件中加载对 DOM 的修改，避免把 JS 脚本放在 head 标签中时 body 元素未加载的问题
        window.addEventListener('load', function() {
            // 附加 CSS 内容
            webCommon.setExtraCSS()

            // 当前页面未设置字体时，修改默认字体为无衬线字体
            if (!document.body.style.fontFamily) {
                document.body.style.fontFamily = '"Helvetica Neue", Helvetica, "PingFang SC", "Hiragino Sans GB", "Microsoft YaHei", 微软雅黑, Arial, sans-serif'
            }
        })
    },

    /**
     * 获取用户信息
     *
     * @returns {{}}
     */
    JS_BRIDGE_GET_USER: function() {
        var cookieData = webCommon.getCookieFields()

        return Object.assign({}, {
            appName: 'ciq3',
            appVersion: 'wechat',
            sysType: 'wechat',
            sysVersion: wechatVersion,
            deviceId: 'WECHAT',
            sessionId: '',
            lang: 'zh',
            domain: webCommon.getApiDomain()
        }, cookieData)
    },

    /**
     * 获取用户资料
     */
    JS_BRIDGE_GET_USER_INFO: function() {
        return webBridgeMethods.JS_BRIDGE_GET_USER_INFO()
    },

    /**
     * 打开指定页面
     *
     * @param params
     */
    JS_BRIDGE_OPEN_PAGE: function(params) {
        var pageName = params.pageName,
            h5Domain = webCommon.getH5Domain()

        switch (pageName) {
            // case 'COURSE':
            //     // 知识库
            // case 'COURSE_SET_DETAIL':
            //     // 课程集
                // location.href = h5Domain + 'api/functions/ciq-only'
                // break
            case 'TASK':
                location.href = h5Domain + 'api/task/list'
                break

            case 'TASK_DETAIL':
                location.href = h5Domain + 'api/task/detail?' + new URLSearchParams(params.data)
                break

            case 'COURSE_QUIZ':
                location.href = h5Domain + 'api/course/quiz-gate?courseId=' + params.data.courseId
                break

            case 'COURSE_PAGE':
                location.href = h5Domain + 'api/course/course-page-container?courseSetId=' + params.data.courseSetId + '&courseId=' + params.data.courseId
                break

            case 'ACHIEVEMENT':
                location.href = h5Domain + 'api/achievement'
                break

            default:
                webBridgeMethods.JS_BRIDGE_OPEN_PAGE(params)
        }
    },

    /**
     * 显示加载动画（不包含文本）
     */
    JS_BRIDGE_OPEN_LOADING: function() {
        webCommon.showLoading()
    },

    /**
     * 隐藏加载动画
     */
    JS_BRIDGE_CLOSE_LOADING: function() {
        webCommon.hideLoading()
    },

    JS_BRIDGE_CLOSE_WINDOW: function() {
        wx.closeWindow();
    },

    JS_BRIDGE_UPLOAD_PIC: function(data) {
        var callback = data.callback

        // 选择图片
        wx.chooseImage({
            count: 1,
            sizeType: ['compressed'],
            success: function(res) {
                var localId = res.localIds[0];

                webCommon.showLoading()

                // 上传图片
                wx.uploadImage({
                    localId: localId, // 需要上传的图片的本地ID，由chooseImage接口获得
                    success: function (res) {
                        var serverId = res.serverId; // 返回图片的服务器端ID

                        ajax.get({
                            url: webCommon.getH5Domain() + 'api/wechat/upload-image?media_id=' + serverId,
                            success: function(data) {
                                webCommon.hideLoading()
                                if (data.status === 200) {
                                    callback(data.data.imgUrl)
                                } else {
                                    alert(data.message)
                                }
                            },
                            error: function() {
                                webCommon.hideLoading()
                            }
                        })
                    },
                    fail: function() {
                        webCommon.hideLoading()
                    }
                });
            }
        })
    },

    /**
     *
     * @param data
     * @returns {string}
     */
    JS_BRIDGE_SHOW_IMAGE: function(data) {
        var imageUrl = data.imageUrl,
            currentPos = data.currentPos

        wx.previewImage({
            urls: imageUrl,
            current: imageUrl[currentPos]
        })
    },

    JS_BRIDGE_SHARE: function(data) {
        wx.showMenuItems({
            menuList: [
                "menuItem:share:appMessage",
                "menuItem:share:timeline"
            ]
        });

        wx.onMenuShareTimeline({
            title: data.title,
            link: data.url,
            imgUrl: data.image,
            success: function() {
                // TODO 添加分享成功回调
            }
        })

        wx.onMenuShareAppMessage({
            title: data.title,
            desc: data.desc,
            link: data.url,
            imgUrl: data.image,
            type: 'link',
            success: function() {}
        })
    },

    /**
     * 高级分享（处理方式同分享接口）
     * @param params
     */
    JS_BRIDGE_PLATFORM_SHARE: function(params) {
        this.JS_BRIDGE_SHARE(params)
    },

    /**
     * 重登录，当用户信息过期时使用
     */
    JS_BRIDGE_SESSION_INVALID: function() {
        webCommon.logout(function() {
            // 关闭窗口
            callBridge('CLOSE_WINDOW');
        });
    },

    /**
     * 事件记录（埋点）
     */
    JS_BRIDGE_EVENTS_STATISTIC: function(params) {
        mixPanel.trackEvent(params.eventName, params.properties)
    }
}

function invoke(key, data) {
    if (typeof bridgeMethods[key] === 'function') {

        var methodsWithoutInit = [
            'INIT',
            'JS_BRIDGE_GET_USER',
            'JS_BRIDGE_GET_USER_INFO',
            'JS_BRIDGE_SET_USER',
            'JS_BRIDGE_OPEN_PAGE',
            'JS_BRIDGE_OPEN_LOADING',
            'JS_BRIDGE_CLOSE_LOADING'
        ]

        if (!isAccessible) {
            return null;
        }

        if (!isWechatReady && methodsWithoutInit.indexOf(key) < 0) {
            callStack.push([
                key,
                data
            ])

            console.info('Wechat invoking is cached')
            return null;
        }

        return bridgeMethods[key](data)
    }

    return null;
}

export default invoke
