/**
 *
 * @author    Jerry Bendy
 * @since     2/26/2018
 */

import callBridge from './callBridge'
import hmacsha1 from 'hmacsha1'
import forceResolveHTTPS from './forceResolveHTTPS'

var noop = function() {};

export default {
    post: function(url, data, options) {
        if (typeof url === 'object' && !data && !options) {
            options = url

        } else {
            Object.assign(options, {
                url: url,
                data: data
            })
        }

        makeRequest('POST', options)
    },

    get: function(url, options) {
        if (typeof url === 'object' && !options) {
            options = url
        } else {
            Object.assign(options, {
                url: url
            })
        }

        makeRequest('GET', options)
    },

    ajax: function(options) {
        makeRequest('', options)
    }
}

function makeRequest(type, params) {
    params = Object.assign({}, {
        type: type || 'GET',
        url: '',
        headers: {},
        data: '',
        withSignature: false,   // 接口是否需要签名认证
        timeout: 0,  // 超时时间，单位毫秒
        withCredentials: false,
        withoutSession: false,  // 不传 session 信息，默认关闭
        success: noop,
        error: noop,
        complete: noop
    }, params)

    var xhr = new XMLHttpRequest();

    // 根据当前页面协议转换传入 URL
    params.url = forceResolveHTTPS(params.url);
    // URL 后面添加 requestId 用于禁止缓存
    params.url += (/\?/.test(params.url) ? '&' : '?') + '__=' + ~~(Math.random() * 1e10);

    xhr.open(params.type, params.url);

    // 超时时间
    if (params.timeout > 0) {
        xhr.timeout = params.timeout;
    }

    // 获取请求基本 header 内容
    if (!params.withoutSession) {
        var userSession = callBridge('GET_USER') || {}
        params.headers = Object.assign({}, {
            appName: userSession.appName || 'ciq3',
            appVersion: userSession.appVersion || '',
            sysType: userSession.sysType || '',
            sysVersion: userSession.sysVersion || '',
            deviceId: userSession.deviceId || '',
            sessionId: userSession.sessionId || '',
            lang: userSession.lang || 'zh'
        }, params.headers)
    }

    // 计算签名
    if (params.withSignature) {
        params.headers.sign = sign(params.data);
    }

    // 跨域传 Cookie
    xhr.withCredentials = params.withCredentials;

    // 生成请求正文
    var _data = null;

    if (params.data) {
        if (params.data instanceof FormData) {
            _data = params.data;

        } else if (typeof params.data === 'object') {
            // 以不区分大小写的方式查找 headers 中是否包含 Content-Type
            // reduce 方法在成功时返回 Content-Type 的实际名称，不存在时返回 false
            var contentTypeName = Object.keys(params.headers).reduce(function(prev, current) {
                return prev ? prev : (current.toLowerCase() === 'content-type' ? current : false)
            }, false)

            if (contentTypeName && /json/i.test(params.headers[contentTypeName])) {
                _data = JSON.stringify(params.data)

            } else {
                params.headers['Content-Type'] = 'application/x-www-form-urlencoded; charset=UTF-8';
                _data = (new URLSearchParams(params.data)).toString();
            }
        } else {
            _data = params.data;
        }
    }

    // 设置 header
    for (var i in params.headers) {
        if (params.headers.hasOwnProperty(i)) {
            xhr.setRequestHeader(i, params.headers[i])
        }
    }

    xhr.send(_data);

    xhr.onreadystatechange = function () {
        if (xhr.readyState === 4) {
            if (xhr.responseText) {
                var result;
                try {
                    result = JSON.parse(xhr.responseText);

                } catch (e) {
                    result = xhr.responseText;
                }

                params.success(result, xhr);
                params.complete(xhr);
            }
        }
    };
    xhr.ontimeout = function (e) {
        params.error(e, xhr);
        params.complete(xhr);
    };
    xhr.onerror = function (e) {
        params.error(e, xhr);
        params.complete(xhr);
    };
}

/**
 * 请求数据签名
 *
 * @param data
 * @returns {string}
 */
function sign(data) {
    // 仅在 data 是对象时调用 Object.keys，修复 iOS8 不支持字符串参数的问题
    if (typeof data !== 'object') {
        return ''
    }
    var str = Object.keys(data).sort().map(function(key) {
        return key + '=' + data[key];
    }).join('&');

    return hmacsha1('tQbPwzcmERdu', str);
}
